package automated;

import static org.junit.Assert.assertEquals;
import org.junit.Test;

public class AutomatedMyListTest {
    public static final int MIN_LENGTH = 0;
    public static final int MAX_LENGTH = 1000;

    public static final int REVERSE_START_RANGE = 0;
    public static final int REVERSE_END_RANGE = 1000;
    
    public static final int SAME_CONTENTS_APPEND_MIN = 0;
    public static final int SAME_CONTENTS_APPEND_MAX = 100;

    public static final int DIFFERENT_CONTENTS_APPEND_START_RANGE = 0;
    public static final int DIFFERENT_CONTENTS_APPEND_END_RANGE = 100;

    public static <A> MyList<A> makeListOfContentsLength(final A contents,
                                                         final int length) {
        MyList<A> result = new EmptyList<A>();
        for (int x = 0; x < length; x++) {
            result = new NonEmptyList<A>(contents, result);
        }
        return result;
    }

    @Test
    public void testLength() {
        for (int length = MIN_LENGTH; length <= MAX_LENGTH; length++) {
            final MyList<Object> list =
                makeListOfContentsLength(new Object(), length);
            assertEquals(length, list.length());
        }
    }

    public static MyList<Integer> makeListOfCountingContents(final int start,
                                                             final int end) {
        MyList<Integer> result = new EmptyList<Integer>();
        for (int current = end; current >= start; current--) {
            result = new NonEmptyList<Integer>(Integer.valueOf(current),
                                               result);
        }
        return result;
    }

    public static MyList<Integer> makeListOfCountingContentsReversed(final int start,
                                                                     final int end) {
        MyList<Integer> result = new EmptyList<Integer>();
        for (int current = start; current <= end; current++) {
            result = new NonEmptyList<Integer>(Integer.valueOf(current),
                                               result);
        }
        return result;
    }

    @Test
    public void testReverse() {
        // move the start of the range, but not the end
        final int end = REVERSE_END_RANGE;
        for (int start = REVERSE_START_RANGE;
             start <= end;
             start++) {
            final MyList<Integer> list = makeListOfCountingContents(start, end);
            final MyList<Integer> expected =
                makeListOfCountingContentsReversed(start, end);
            assertEquals(expected, list.reverse());
        }
    }
    
    @Test
    public void testAppendSameContents() {
        final Object contents = new Object();
        for (int length1 = SAME_CONTENTS_APPEND_MIN;
             length1 <= SAME_CONTENTS_APPEND_MAX;
             length1++) {
            for (int length2 = SAME_CONTENTS_APPEND_MIN;
                 length2 <= SAME_CONTENTS_APPEND_MAX;
                 length2++) {
                final MyList<Object> list1 =
                    makeListOfContentsLength(contents, length1);
                final MyList<Object> list2 =
                    makeListOfContentsLength(contents, length2);
                final MyList<Object> expected =
                    makeListOfContentsLength(contents, length1 + length2);
                assertEquals(expected, list1.append(list2));
                assertEquals(expected, list2.append(list1));
            }
        }
    }

    @Test
    public void testAppendDifferentContents() {
        // -pick a low end and a high end, forming a range
        // -divide this range into two pieces, in as many ways as possible
        // -ensure that appending these two pieces results in the same range

        // -Move the start of the range, but keep the end of the range fixed
        final int endRange = DIFFERENT_CONTENTS_APPEND_END_RANGE;
        for (int startRange = DIFFERENT_CONTENTS_APPEND_START_RANGE;
             startRange <= endRange;
             startRange++) {
            final MyList<Integer> expected =
                makeListOfCountingContents(startRange, endRange);
            // now we have a start and end of the range
            // for each possible point, divide into two segments
            for (int divisionPoint = startRange;
                 divisionPoint <= endRange;
                 divisionPoint++) {
                final MyList<Integer> left =
                    makeListOfCountingContents(startRange, divisionPoint);
                final MyList<Integer> right =
                    makeListOfCountingContents(divisionPoint + 1, endRange);
                assertEquals(expected, left.append(right));
            }
        }
    }
}
